package mscJNeuralNet.examples;

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import mscJNeuralNet.connectors.INetConnector;
import mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector;
import mscJNeuralNet.gui.superComponents.JEnvironmentPanel;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.patterns.Patterns;
import mscJNeuralNet.trainer.NetTrainer;
import mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm;
import mscJNeuralNet.trainingAlgorithms.RProp;
import observerPattern.Observable;
import observerPattern.Observer;
import formats.Formats;

/**
 * Beispielprogramm fr den Einsatz der GUI-Komponenten unter der Klasse
 * {@link mscJNeuralNet.gui.superComponents.JEnvironmentPanel}.
 * <p/>
 * Created on 14.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 14.06.2004
 */
public class TestJEnvironmentPanel  implements Observer{
	private Net mNet;
	private NetTrainer mNetTrainer;
	
	public TestJEnvironmentPanel(){
		try{
			Patterns lPat = Patterns.load("alle+tonleiter-smp.txt");
			
			int [] layerSizes = {15, 8, 8, 3};
			mNet = new Net(layerSizes);
			INetConnector lRandNetConnector = new RandomSymmetryBreakingNetConnector();
			lRandNetConnector.connectNet(mNet, null);
			
			mNetTrainer = new NetTrainer();
			mNetTrainer.setTrainingPatterns(lPat);
			mNetTrainer.setTargetErrorType(NetPerformanceStatistics.ERRORTYPE_averageSumOfSquaredError);
			mNetTrainer.setTargetError(.01);
			mNetTrainer.setTargetCycles(1000);
			
			INetTrainingAlgorithm lAlgo = new RProp();
			lAlgo.setNet(mNet);
			mNetTrainer.setTrainingAlgorithm(lAlgo);
			
			mNetTrainer.getObserverManager().addObserver(this);
			
			JFrame fMain = new JFrame();
			fMain.addWindowListener(
					new WindowAdapter() {
						public void windowClosing(WindowEvent e){
							System.exit(0);
						}
					}
			);
			
			Container content = fMain.getContentPane();
			content.setLayout(new FlowLayout());
			
			JEnvironmentPanel lPEnvironment = new JEnvironmentPanel();
			lPEnvironment.setNetTrainer(mNetTrainer);
			content.add(lPEnvironment);
			
			fMain.pack();
			fMain.setVisible(true);
			
			System.out.println(Formats.format(mNet.getWeights()));
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		if (pObservable == mNetTrainer){
			if (mNetTrainer.hasFinished()){
				System.out.println(
					NetPerformanceReporter.getNetPerformance(
						mNetTrainer.getLastCalcualtedNetStatistics(), 
						mNetTrainer.getTrainingAlgorithm().getCycle()));
				System.out.println(Formats.format(mNet.getWeights()));
			}
		}
	}
	
	public static void main(String[] args) {
		new TestJEnvironmentPanel();
	}

}
