package mscJNeuralNet.examples;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter;
import mscJNeuralNet.connectors.INetConnector;
//import mscJNeuralNet.connectors.RandomNetConnector;
import mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector;
//import mscJNeuralNet.connectors.WesselsBarnardNetConnector;
import mscJNeuralNet.trainingAlgorithms.*;


/**
 * Beispiel zum Trainieren eines Netzes mit der dreidimensionalen Xor-Funktion
 * durch die verschiedenen Lernverfahren. 
 * <p/>
 * Created on 31.05.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 31.05.2004
 */
public class TestTrainingAlgorithmsTrippleXOR {

	public static void main(String[] args) {
		double [][] trippleXORtrainInput = {
					{-1,-1,-1}, {-1,-1,1},
					{-1,1,-1}, {-1,1,1},
					{1,-1,-1}, {1,-1,1},
					{1,1,-1}, {1,1,1}};
		double [][] trippleXORtrainOutput = {
					{-1},{1},
					{1},{-1},
					{1}, {-1},
					{-1}, {1}};
		
		int [] layerSizesTrippleXOr = {3, 3, 1};
		
		// alpha, momentum
		double [] momentum_1_bpg = {0.3, 0.4}; // best performance on tripplexor
		
		// momentum, alphaplus, alphaminus, alphainit
		double [] momentum_1_ssab = {0.0}; // best performance on tripplexor

		// alpha, max growth rate
		double [] momentum_1_qprop = {0.3};
		
		Net lTrippleXorNet = new Net(layerSizesTrippleXOr);
		
		INetTrainingAlgorithm lAlgoBpgMom = new BackpropagationMomentum();
		INetTrainingAlgorithm lAlgoSSAB = new SuperSABMomentum();
		INetTrainingAlgorithm lAlgoRProp = new RProp();
		INetTrainingAlgorithm lAlgoQProp = new QuickProp();
				
		INetConnector lSymmNetConnector = new RandomSymmetryBreakingNetConnector();
		
		try{
			lAlgoBpgMom.setLearningParameters(momentum_1_bpg);
			NetPerformanceReporter.getQualityReport(lTrippleXorNet, lAlgoBpgMom, trippleXORtrainInput, trippleXORtrainOutput, 50, 100, null, lSymmNetConnector);
			
			lAlgoSSAB.setLearningParameters(momentum_1_ssab);
			NetPerformanceReporter.getQualityReport(lTrippleXorNet, lAlgoSSAB, trippleXORtrainInput, trippleXORtrainOutput, 50, 100, null, lSymmNetConnector);
			
			NetPerformanceReporter.getQualityReport(lTrippleXorNet, lAlgoRProp, trippleXORtrainInput, trippleXORtrainOutput, 50, 100, null, lSymmNetConnector);
			
			lAlgoQProp.setLearningParameters(momentum_1_qprop);
			NetPerformanceReporter.getQualityReport(lTrippleXorNet, lAlgoQProp, trippleXORtrainInput, trippleXORtrainOutput, 50, 100, null, lSymmNetConnector);
		}
		catch (Exception e){
			e.printStackTrace();
		}
		System.exit(0);
	}
}
