 package mscJNeuralNet.examples;

import mscJNeuralNet.connectors.INetConnector;
import mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.net.PatternDoesNotMatchNetException;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisticsCalculator;
import mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm;
import mscJNeuralNet.trainingAlgorithms.RProp;


/**
 * Beispiel fr das Trainineren eines Netzes mit dem dreidimensionalen Xor-Problem.
 * <p/>
 * Created on 02.08.2004.
 * @author <a href="http://www.graviton.de">M. Serhat Cinar</a>
 * @version 24.12.2004
 * @see mscJNeuralNet.net.Net
 * @see mscJNeuralNet.connectors.INetConnector
 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm
 * @see mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics
 * @see mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisticsCalculator
 */
//Trainieren eines MLP
public class TestTrippleXorLearn{

	public static void main(String argv[]){
		// 1. Erzeugen der bentigten Klassen
		
		// MLP {@link mscJNeuralNet.net.Net}
		// MLP mit der gewnschten Schichtstruktur erstellen:
		// Eingabeschicht: 3 Neuronen
		// 1. Verdeckte Schicht: 3 Neuronen
		// Ausgabeschicht: 1 Neuron
		int [] layerSizesTrippleXOr = {3, 3, 1};
		//* {@link mscJNeuralNet.net.Net} myNet = new Net(layerSizesTrippleXOr);
		Net myNet = new Net(layerSizesTrippleXOr);

		// BIAS wurde automatisch bercksichtigt.

		// {@link mscJNeuralNet.connectors.INetConnector}
		// Diese Klasse wird zum Initialisieren der Kantengewichte bentigt
		INetConnector lNetConnectionAlgo = new RandomSymmetryBreakingNetConnector();
		
		// {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm}
		// Diese Klasse wird zum Trainieren eines MLP bentigt
		INetTrainingAlgorithm lNetTrainAlgo = new RProp();
		
		
		// 2. Initialisieren des Netzes
		
		// MLP + {@link mscJNeuralNet.connectors.INetConnector}
		// Mit der Instanz von {@link mscJNeuralNet.connectors.INetConnector} inititalisieren.
		// Die Klasse {@link mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector} bentigt
		// keine eigenen Kontrollparameter und wird daher mit dem Wert null aufgerufen.
		lNetConnectionAlgo.connectNet(myNet, null);
		
		// Nun ist das Netz verbunden und initialisiert.
		
		
		// 3. Trainieren des Netzes
		
		// TRAININGSDATEN
		// Trainingsdaten bereitstellen
		double [][] lTrippleXORtrainInput = {
					{-1,-1,-1}, {-1,-1,1},
					{-1,1,-1}, {-1,1,1},
					{1,-1,-1}, {1,-1,1},
					{1,1,-1}, {1,1,1}};
		double [][] lTrippleXORtrainOutput = {
					{-1},{1},
					{1},{-1},
					{1}, {-1},
					{-1}, {1}};
		
		// {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm}
		// Lernverfahren {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm} mit Netz verbinden.
		lNetTrainAlgo.setNet(myNet);
		
		
		try{
			// 50 TRAININGSZYKLEN
			while (lNetTrainAlgo.getCycle()<250){
				Net.train(lTrippleXORtrainInput, lTrippleXORtrainOutput, lNetTrainAlgo);
			}
			
			// {@link mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisiticsCalculator}
			// {@link mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisitics}
			// Berechnen der aktuellen Netzperformanz (Soll-Ist Fehler)
			NetPerformanceStatistics lNetPerformance = NetPerformanceStatisticsCalculator.calculateErrors(myNet, lTrippleXORtrainInput, lTrippleXORtrainOutput);
			
			// {@link mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter}
			// Formatierte Ausgabe der Netzperformanz
			System.out.println(NetPerformanceReporter.getNetPerformance(lNetPerformance, lNetTrainAlgo.getCycle()));
		}
		//* catch({@link mscJNeuralNet.patterns.PatternDoesNotMatchNetException} e){
		catch(PatternDoesNotMatchNetException e){
			// Wird aufgerufen, wenn die Lerndatenstze mehr Eingabe-/Ausgabewerte 
			// enthalten als Eingabe-/Ausgabeneuronen im MLP vorhanden sind.
			e.printStackTrace();
		}
	}	
}