package mscJNeuralNet.gui;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import mscJNeuralNet.gui.subComponents.JLabeledTextField;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.trainer.NetTrainer;
import observerPattern.Observable;
import observerPattern.Observer;
import formats.Formats;

/**
 * Grafische Komponente zur Kontrolle des Trainings.<p/>
 * <img src="../../images/jnettrainercontrol.gif"><p/>
 * 
 * Created on 06.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 06.06.2004
 */
public class JNetTrainerControl extends JPanel implements Observer{
	private static final ResourceBundle smMSCResourceBundle = ResourceBundle.getBundle("mscJNeuronalNet_Text");
	
	private JPanel mPMain;
	private JComboBox mCBErrorTypes; 
	private JLabeledTextField mLTfActualError;
	private JLabeledTextField mLTfTargetError;
	private JLabeledTextField mLTfActualCycles;
	private JLabeledTextField mLTfTargetCycles;
	private JCheckBox mChBUseRandomizedPatternOrder;
	
	// TODO: was passiert, wenn man vor dem trainieren etwas testen muss? z.b. ob lerndaten verfgbar sind
	private JButton mBStart;
	private NetTrainer mNetTrainer;
	
	public JNetTrainerControl(){
		super();
		setBorder(
			BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.BLACK),
				smMSCResourceBundle.getString("TRAINER")
			)
		);
		mPMain = new JPanel();
		mPMain.setLayout(new GridLayout(7, 1));
		
		mCBErrorTypes = new JComboBox(NetPerformanceStatistics.STR_errorTypes_short);
		mCBErrorTypes.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					int lSelectedErrorType = mCBErrorTypes.getSelectedIndex();
					if (lSelectedErrorType != getNetTrainer().getTargetErrorType())
						getNetTrainer().setTargetErrorType(lSelectedErrorType);
				}
			}
		);
		mPMain.add(mCBErrorTypes);
		
		mLTfActualCycles = new JLabeledTextField(5);
		mLTfActualCycles.getLabel().setText(smMSCResourceBundle.getString("ACTUAL_CYCLE")+":");
		mLTfActualCycles.getTextField().setEditable(false);
		mLTfActualCycles.getObserverManager().addObserver(this);
		mPMain.add(mLTfActualCycles);
		
		mLTfTargetCycles = new JLabeledTextField(5);
		mLTfTargetCycles.getLabel().setText(smMSCResourceBundle.getString("MAX_CYCLES")+":");
		mLTfTargetCycles.getObserverManager().addObserver(this);
		mPMain.add(mLTfTargetCycles);
		
		mLTfActualError = new JLabeledTextField(5);
		mLTfActualError.getLabel().setText(smMSCResourceBundle.getString("ACTUAL_ERROR")+":");
		mLTfActualError.getTextField().setEditable(false);
		mLTfActualError.getObserverManager().addObserver(this);
		mPMain.add(mLTfActualError);
		
		mLTfTargetError = new JLabeledTextField(5);
		mLTfTargetError.getLabel().setText(smMSCResourceBundle.getString("MAX_ERROR")+":");
		mLTfTargetError.getObserverManager().addObserver(this);
		mPMain.add(mLTfTargetError);
		
		mChBUseRandomizedPatternOrder = new JCheckBox(smMSCResourceBundle.getString("RANDOMIZE_PATTERN_ORDER"));
		mChBUseRandomizedPatternOrder.addItemListener(
			new ItemListener(){
				public void itemStateChanged(ItemEvent e) {
					if (getNetTrainer() != null){
						if (e.getStateChange() == ItemEvent.DESELECTED){
							getNetTrainer().setUseRandomizedPatternOrder(false);
						}
						else if (e.getStateChange() == ItemEvent.SELECTED){
							getNetTrainer().setUseRandomizedPatternOrder(true);
						}
					}
				}
			}
		);
		mPMain.add(mChBUseRandomizedPatternOrder);
		
		mBStart = new JButton(smMSCResourceBundle.getString("START_TRAINING"));
		mBStart.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent arg0) {
					if (getNetTrainer() != null){
						if (getNetTrainer().isRunning()) getNetTrainer().stop();
						else getNetTrainer().start();
					}
				}
			} 
		);
		mPMain.add(mBStart);
		add(mPMain);
	}
	
	public void setNetTrainer(NetTrainer pNetTrainer){
		if (pNetTrainer != mNetTrainer && pNetTrainer != null){
			if (mNetTrainer != null) mNetTrainer.getObserverManager().removeObserver(this);
			mNetTrainer = pNetTrainer;
			mNetTrainer.getObserverManager().addObserver(this);
			updateNetTrainerViews();
		}
	}
	
	public NetTrainer getNetTrainer(){
		return mNetTrainer;
	}
	
	private void updateNetTrainerViews(){
		mLTfTargetError.getLabel().setText(smMSCResourceBundle.getString("MAX")+" "+mNetTrainer.getTargetErrorTypeString()+":");
		mLTfTargetError.getTextField().setText(Formats.format(mNetTrainer.getTargetError(), 6));
		mLTfActualError.getLabel().setText(smMSCResourceBundle.getString("ACTUAL")+" "+mNetTrainer.getTargetErrorTypeString()+":");
		mLTfActualError.getTextField().setText(Formats.format(mNetTrainer.getLastCalculatedError(), 6));
		if (mNetTrainer.getTrainingAlgorithm() == null)
			mLTfActualCycles.getTextField().setText("0");
		else 
			mLTfActualCycles.getTextField().setText(Formats.format(mNetTrainer.getTrainingAlgorithm().getCycle()));
		mLTfTargetCycles.getTextField().setText(Formats.format(mNetTrainer.getTargetCycles()));
		mCBErrorTypes.setSelectedIndex(mNetTrainer.getTargetErrorType());
		if (mNetTrainer.isUsingRandomizedPatternOrder()) mChBUseRandomizedPatternOrder.setSelected(true);
		else mChBUseRandomizedPatternOrder.setSelected(false);
		if (mNetTrainer.isRunning()) mBStart.setText(smMSCResourceBundle.getString("STOP_TRAINING"));
		else mBStart.setText(smMSCResourceBundle.getString("START_TRAINING"));
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		if (pObservable == mLTfTargetError){
			double lGuiTargetError = Double.parseDouble(mLTfTargetError.getTextField().getText());
			if (lGuiTargetError != mNetTrainer.getTargetError())
				mNetTrainer.setTargetError(lGuiTargetError);
		}
		else if (pObservable == mLTfTargetCycles){
			int lGuiTargetCycles = Integer.parseInt(mLTfTargetCycles.getTextField().getText());
			if (lGuiTargetCycles != mNetTrainer.getTargetCycles())
				mNetTrainer.setTargetCycles(lGuiTargetCycles);
		}
		else if (pObservable == mNetTrainer){
			updateNetTrainerViews();
		}
	}
}
