package mscJNeuralNet.gui.subComponents;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.ResourceBundle;

import javax.swing.JPanel;

import mscJNeuralNet.net.Net;
import observerPattern.Observable;
import observerPattern.Observer;
import formats.Formats;

/**
 * Eine Darstellung der Gewichtsstruktur fr ein Netz.
 * Diese Komponente wird von der Komponente {@link mscJNeuralNet.gui.JNetWeightView} benutzt.
 * <p/>
 * <img src="../../../images/jweightview.gif"/><p/>
 * Created on 26.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 27.06.2004
 * @see mscJNeuralNet.gui.JNetWeightView
 */
public class JWeightView extends JPanel implements Observer{
	private static final ResourceBundle smMSCResourceBundle = ResourceBundle.getBundle("mscJNeuronalNet_Text");
	
	private Net mNet;
	private static int smMinWidth = 7;
	private static int smMinHeight = 7;
	private int mNeuronsAmount;
	private int mNeuronWidth;
	private int mNeuronHeight;
	
	private static final float smBlue = 0;
	/** Weight value, at which the maximum color amount will be applied */
	private static final double smColorScaleFactor = 2D;
	private Color mMouseOverColor;
	private static final Color smMouseOverValue = Color.WHITE;
	private static final Color smMouseOverZero = Color.BLUE;
	private static final Color smMouseOut = Color.BLACK;
	// private static final Dimension smDefaultSize = new Dimension(680, 280); 
	
	private int mOutputtingMouseOver = 0;
	private int mInputtingMouseOver = 20;
	private int mOutputtingMouseOverNew = 0;
	private int mInputtingMouseOverNew = 0;
	
	private int mStartDrawingAtOutputting;
	private int mStartDrawingAtInputting;
	private int mEndDrawingAtOutputting;
	private int mEndDrawingAtInputting;
	private int mOutputtingAll;
	private int mInputtingAll;
	private String mOutputtingString;
	private String mInputtingString;
	
	private boolean mShowMouse = false;
	
	/**
	 * Erzeugt eine Darstellung der Gewichtsstruktur fr ein Netz.
	 */
	public JWeightView(){
		super();
		setBackground(Color.WHITE);
		// setPreferredSize(smDefaultSize);
		addMouseMotionListener(
			new MouseMotionAdapter(){
				public void mouseMoved(MouseEvent e){
					mInputtingMouseOverNew = (int)(e.getX()/mNeuronWidth);
					if (mInputtingMouseOverNew >= mInputtingAll) mInputtingMouseOverNew = mInputtingAll-1;
					mOutputtingMouseOverNew = (int)(e.getY()/mNeuronHeight);
					if (mOutputtingMouseOverNew >= mOutputtingAll) mOutputtingMouseOverNew = mOutputtingAll-1;
					if (mOutputtingMouseOverNew != mOutputtingMouseOver || mInputtingMouseOverNew != mInputtingMouseOver){
						mOutputtingMouseOver = mOutputtingMouseOverNew; 
						mInputtingMouseOver = mInputtingMouseOverNew;
						
						if (mOutputtingMouseOver == 0)
							mOutputtingString = smMSCResourceBundle.getString("BIAS");
						else if (mOutputtingMouseOver <= mNet.getInputLayerSize())
							mOutputtingString = smMSCResourceBundle.getString("INPUT_NEURON")+" "+(mOutputtingMouseOver+mStartDrawingAtOutputting);
						else mOutputtingString = Integer.toString(mOutputtingMouseOver+mStartDrawingAtOutputting);
		
						if (mInputtingMouseOver >= mInputtingAll-mNet.getOutputLayerSize())
							mInputtingString = smMSCResourceBundle.getString("OUTPUT_NEURON")+" "+(mInputtingMouseOver-(mInputtingAll-mNet.getOutputLayerSize())+1);
						else mInputtingString = Integer.toString(mInputtingMouseOver+mStartDrawingAtInputting);

						if (mNet.getWeights()[mOutputtingMouseOver+mStartDrawingAtOutputting][mInputtingMouseOver+mStartDrawingAtInputting]==0) mMouseOverColor = smMouseOverZero;
						else mMouseOverColor = smMouseOverValue;
						setToolTipText(mOutputtingString+" -> "+mInputtingString+" = "+Formats.format(mNet.getWeights()[mOutputtingMouseOver+mStartDrawingAtOutputting][mInputtingMouseOver+mStartDrawingAtInputting], 6));
						repaint();
					}
				}
			}
		);
		addMouseListener(
			new MouseAdapter(){
				public void mouseEntered(MouseEvent arg0) {
					mShowMouse = true;
					repaint();
				}

				public void mouseExited(MouseEvent arg0) {
					mShowMouse = false;
					repaint();
				}
			}
		);
	}
	
	/**
	 * Legt das Netz fest, welches durch diese Komponente dargestellt werden soll.
	 * 
	 * @param pNet Das Netz, das durch diese Komponente dargestellt werden soll.
	 */
	public void setNet(Net pNet){
		if (mNet != null) mNet.getObserverManager().removeObserver(this);
		mNet = pNet;
		mNet.getObserverManager().addObserver(this);
//		Log.getInstance().writeln(Formats.format(mNet.getWeights()));
		calculateView();
	}
	
	/**
	 * Liefert das Netz zurck, welches durch diese Komponente dargestellt wird.
	 * 
	 * @return Das Netz, welches durch diese Komponente dargestellt wird.
	 */
	public Net getNet(){
		return mNet;
	}
	
	private void calculateView(){
		mNeuronsAmount = mNet.getAmountNeurons();
		
		mStartDrawingAtOutputting = 0;
		mStartDrawingAtInputting = 1+mNet.getInputLayerSize();
		mEndDrawingAtOutputting = mNeuronsAmount-mNet.getOutputLayerSize();
		mEndDrawingAtInputting = mNeuronsAmount;
		mOutputtingAll = mEndDrawingAtOutputting-mStartDrawingAtOutputting;
		mInputtingAll = mEndDrawingAtInputting-mStartDrawingAtInputting;
		
		mNeuronWidth = Math.max(smMinWidth, (int)(getWidth()/mInputtingAll));
		mNeuronHeight = Math.max(smMinHeight, (int)(getHeight()/mOutputtingAll));

		lXPixelMax = mNeuronWidth*mInputtingAll;
		lYPixelMax = mNeuronHeight*mOutputtingAll;
		
		Dimension lPreferredSize = new Dimension();
		lPreferredSize.width = Math.max(getWidth(), lXPixelMax);
		lPreferredSize.height = Math.max(getHeight(), lYPixelMax);
		setPreferredSize(lPreferredSize);
		revalidate();
	}
	
	private int lXPixel, lInputting;
	private int lYPixel, lOutputting;
	private int lXPixelMax;
	private int lYPixelMax;
	private double [][] lWeights;
	private float lX;
	private double lActualWeightValue;
	private float lRed, lGreen;
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics pG){
		super.paintComponent(pG);
		if (mNet != null){
			lInputting = mStartDrawingAtInputting;
			lWeights = mNet.getWeights();
			for (lXPixel = 0; lXPixel<lXPixelMax; lXPixel+=mNeuronWidth){
				lOutputting = mStartDrawingAtOutputting;
				for (lYPixel = 0; lYPixel<lYPixelMax; lYPixel+=mNeuronHeight){
					lActualWeightValue = lWeights[lOutputting][lInputting];
					if (lActualWeightValue != 0D){
						lX = (float) (lActualWeightValue*smColorScaleFactor);
						if (lActualWeightValue < 0D){
							lX = Math.abs(lX);
							if (lX > 1) lX = 1;
							lRed = lX;
							lGreen = 0;
						}
						else{
							if (lX > 1) lX = 1;
							lRed = 0;
							lGreen = lX;
						}
						pG.setColor(new Color(lRed, lGreen, smBlue));
						pG.fillRect(lXPixel, lYPixel, mNeuronWidth, mNeuronHeight);
						pG.setColor(smMouseOut);
						pG.drawRect(lXPixel, lYPixel, mNeuronWidth, mNeuronHeight);
					}
					lOutputting++;
				}
				lInputting++;
			}
			if (mShowMouse){
				pG.setColor(mMouseOverColor);
				pG.drawRect(mNeuronWidth*mInputtingMouseOver, mNeuronHeight*mOutputtingMouseOver, mNeuronWidth, mNeuronHeight);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		if (pObservable == mNet && mNeuronsAmount != mNet.getAmountNeurons()){
			mNeuronsAmount = mNet.getAmountNeurons();
			calculateView();
		}
		else if (isVisible()) repaint();
	}
}
