package mscJNeuralNet.gui.superComponents;

import java.util.ResourceBundle;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import mscJNeuralNet.gui.JNetErrorGraphView;
import mscJNeuralNet.gui.JNetInfoView;
import mscJNeuralNet.gui.JNetTrainerControl;
import mscJNeuralNet.gui.JNetTrainingAlgorithmControl;
import mscJNeuralNet.gui.JNetWeightView;
import mscJNeuralNet.trainer.NetTrainer;

/**
 * Ein Panel, dass die wichtigsten GUI-Komponenten der mscJNeuralNet-API
 * einsetzt. Die einzelnen Komponenten werden durch eine TabbelPane
 * zusamengefgt.
 * <p/>
 * Created on 14.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 14.06.2004
 */
public class JEnvironmentPanel extends JPanel{
	private static final ResourceBundle smMSCResourceBundle = ResourceBundle.getBundle("mscJNeuronalNet_Text");
	
	private JTabbedPane mTPMain;
	private JNetErrorGraphView mErrorGraph;
	private JNetInfoView mInfoView;
	private JNetTrainerControl mTrainerControl;
	private JNetTrainingAlgorithmControl mTrainingAlgoControl;
	private JNetWeightView mWeightView;
	
	public JEnvironmentPanel(){
		super();
		
		mErrorGraph = new JNetErrorGraphView();
		mInfoView = new JNetInfoView();
		mTrainerControl = new JNetTrainerControl();
		mTrainingAlgoControl = new JNetTrainingAlgorithmControl();
		mWeightView = new JNetWeightView();
		
		mTPMain = new JTabbedPane(JTabbedPane.TOP);
		
		JPanel lPTraining = new JPanel();
		lPTraining.add(mTrainerControl);
		lPTraining.add(mErrorGraph);
		
		mTPMain.addTab(smMSCResourceBundle.getString("NET_INFO"), mInfoView);
		mTPMain.addTab(smMSCResourceBundle.getString("WEIGHT_STRUCTURE"), mWeightView);
		mTPMain.addTab(smMSCResourceBundle.getString("TRAINING"), lPTraining);
		mTPMain.addTab(smMSCResourceBundle.getString("TRAINING_ALGORITHM"), mTrainingAlgoControl);
		
		add(mTPMain);
	}
	
	/**
	 * Der NetTrainer, der in dieser Komponente benutzt werden soll.
	 * 
	 * @param pNetTrainer NetTrainer, der in dieser Komponente benutzt werden soll.
	 */
	public void setNetTrainer(NetTrainer pNetTrainer){
		mInfoView.setNet(pNetTrainer.getTrainingAlgorithm().getNet());
		mErrorGraph.setNetTrainer(pNetTrainer);
		mWeightView.setNet(pNetTrainer.getTrainingAlgorithm().getNet());
		mTrainerControl.setNetTrainer(pNetTrainer);
		mTrainingAlgoControl.setTrainingAlgorithm(pNetTrainer.getTrainingAlgorithm());
	}
	
	/**
	 * Liefert die TabbedPane dieser Komponente.
	 * 
	 * @return TabbedPane dieser Komponente.
	 */
	public JTabbedPane getTabbedPane(){
		return mTPMain;
	}
}
