package observerPattern;

/**
 * Die Schnittstelle Observer ist von jeder Klasse, 
 * die von einer beobachtbaren Klasse (Observable) aus benachrichtigt 
 * werden mchte, zu implementieren. Diese Schnittstelle 
 * umfasst lediglich die Methode public void notify(Observable pObservable).<br/>
 * Um von einem Observable benachrichtigt werden zu knnen, muss der Observer noch sein
 * Interesse anmelden.<br/>
 * Durch folgende einfachen Schritte kann eine Klasse Observer implementieren:<br/>
 * <dir>
 * 	<li>Die Schnittstelle Observer implementieren:<br/>
 * 	  <code>public class myObserver impements Observer...</code>
 *  </li>
 *  <li>Beim Observable Interesse anmelden:<br/>
 * 	  <code>theObservable.getObserverManager().addObserver(this);</code>
 *  </li>
 *  <li>Methode public void notify(Observable pObservable) implementieren<br/>
 *    <code>
 * 		public void notify(Observable pObservable){
 * 			if (pObservable == theObserver) dosomething();
 * 		}</code>
 *  </li>
 * </dir>
 * <pre>
 * <p/>
 * Created on 21.04.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 21.04.2004
 * @see observerPattern.Observable
 */
public interface Observer {
	/**
	 * ber diese Mehtode wird der Observer von einem Observable benachrichtigt.
	 * 
	 * @param pObservable Der Observable, der diesen Observer benachrichtigen mchte.
	 * @see observerPattern.Observable
	 */
	public void notify(Observable pObservable);
}
